/**********************************************************************

	Command:  CHPROP -- Change Property of Selected Object

	Programmed by Kang, Hsin-Min.
	Release at Jun 21, 1997
	Updated at Sep 05, 1997 Fix a Ltype Mismatch bug
	Updated at Sep 13, 1997 Add [Help] button
	Revised at Apr 22, 1998 Include Japenese Text in SJIS codes

	Copyright (c) 1997, 1998 TCAM Development House, Taipei
	All rights Reserved.

	Status: Standard Command, with full support.

	Note:

		Linetype ID read from entities:

			-1, BYBLOCK
			 0, BYLAYER

***********************************************************************/
int	IsWin;
int	nSupport=identify("wnd_resize");

SLIST	slist;		// Selected objects
int	nObject;	// Number of Object in slist
int	nLayerID;	// Current Layer ID, 0-255
int	nLtypeID;	// Current Linetype ID,
int	nColorID;	// Current Color ID
#define DOLAYER 1
#define DOLTYPE 2
#define DOCOLOR 4
int	nControl=0;
//
int	nMain[6]={4,12,8,74,0x3271,0x8000};
int	bChLayer;	// Check Box of Change Layer
int	bChColor;	// Check Box of Change Color
int	bChLtype;	// Check Box of Change Ltype
int	bLayer; 	// Button to change Layer setting
int	bColor; 	// Button to change Color setting
int	bLtype; 	// Button to change Ltype setting
int	bOK;		// Button to do the change
int	bCancel;
int	bPick;
int	bResel;
int	bHelp=0;
//
char	szText[20][40]; // Our display text buffer, see SetMsgText()
			// at the end of this file
//
COMMAND Chprop()
{
	int	nAgain=0;
	IsWin = 1;
	switch(1) {  // Test if we are in TwinCAD ...
	  case !identify("os_type"):
	  case (os_type()<0):	// Must be in DOS version!
		IsWin = 0;
		break;
	}
	SetMsgText();
	while (nAgain=DoChprop(nAgain));
}
//
//	Real job here
//
DoChprop(int agn)
{
	int	nBtn;
	slist = getesel();
	if ( (nObject=ent_count(slist)) == 0 ) {
		if ( !agn )	// Is it the first time selection ?
			return 0;
	}
	CheckObject();
	OpenDialog();
	UpdateDialog();
	for(;;) {
		nBtn = wnd_gbtn(0);
		switch(nBtn) {
		  case -3:
		  case -2:
		  case -1:
		  case	0:
			CloseDialog();
			return 0;
		  case bLayer:
			nLayerID = SelLayer(nLayerID,bLayer);
			UpdateLayer();
			if ( nColorID < 0 )
				UpdateColor();
			if ( nLtypeID < 0 )
				UpdateLtype();
			nControl |= DOLAYER;
			wnd_fmark(bChLayer,0,(nControl&DOLAYER)?0x8000:0x80ff);
			continue;
		  case bLtype:
			nLtypeID = ltypeid(getltype(nLtypeID-1,0).name);
			UpdateLtype();
			nControl |= DOLTYPE;
			wnd_fmark(bChLtype,0,(nControl&DOLTYPE)?0x8000:0x80ff);
			continue;
		  case bColor:
			nColorID = getcolor(nColorID);
			UpdateColor();
			nControl |= DOCOLOR;
			wnd_fmark(bChColor,0,(nControl&DOCOLOR)?0x8000:0x80ff);
			continue;
		  case bChLayer:
			nControl ^= DOLAYER;
			if ( nLayerID == -1 ) {
				nControl &= ~DOLAYER;
				printf(szText[0]/*"\n## No specific layer to change!"*/);
				beep();
			}
			wnd_fmark(bChLayer,0,(nControl&DOLAYER)?0x8000:0x80ff);
			continue;
		  case bChLtype:
			nControl ^= DOLTYPE;
			if ( nLtypeID == -3 ) {
				nControl &= ~DOLTYPE;
				printf(szText[1]/*"\n## No specific linetype to change!"*/);
				beep();
			}
			wnd_fmark(bChLtype,0,(nControl&DOLTYPE)?0x8000:0x80ff);
			continue;
		  case bChColor:
			nControl ^= DOCOLOR;
			if ( nColorID == -3 ) {
				nControl &= ~DOCOLOR;
				printf(szText[2]/*"\n## No specific color to change!"*/);
				beep();
			}
			wnd_fmark(bChColor,0,(nControl&DOCOLOR)?0x8000:0x80ff);
			continue;
		  case bResel:
			CloseDialog();
			return 1;	// Need to re-enter this routine!
		  case bPick:
			CloseDialog();
			e1 = gete(szText[3]/*"\nPick property from entity: "*/);
			if ( ent_ok(e1) ) {
				i0 = ent_read(e1).layer;
				if ( i0 != nLayerID ) {
					nLayerID = i0;
					nControl |= DOLAYER;
				}
				i0 = ent_read(e1).color;
				if ( i0 != nColorID ) {
					nColorID = i0;
					nControl |= DOCOLOR;
				}
				i0 = ent_read(e1).ltypeno;
				if ( i0 != nLtypeID ) {
					nLtypeID = i0;
					nControl |= DOLTYPE;
				}
			}
			OpenDialog();
			UpdateDialog();
			continue;
		  case bCancel:
			CloseDialog();
			return 0;
		  case bOK:
			CloseDialog();
			DoChange();
			return 0;
		  case bHelp:
			show_help("","CHPROP");
			continue;
		  default:
			continue;
		}
	}
}
//
//	SelLayer() -- Select a layer
//
int	SelLayer(int lid,int bid)
{
	char	name[256][24];
	int	i,j;
	LAYER	mylayer;

	if ( identify("getlayer") )     // If we have the support,
		return getlayer(lid);	//    call it directly

	// Otherwise, we implement our own simple one

	mylayer = layer(0);
	i = 0;
	j = -1;
	while(mylayer.name!="") {
		if ( layerid(mylayer.name)==lid )
			j = i;
		name[i] = (j==i)?"* ":"  ";
		name[i] += mylayer.name;
		i++;
		mylayer = tbl_next(mylayer);
	}
	name[i] = "";
	i=wnd_fgeto(bid,name,0x2f1,8,j);
	return layerid(name[i]-left$(name[i],2));
}
//
//	DoChange() -- Change properties of selected objects.
//
void	DoChange()
{
	int	i;
	ENTDATA data;
	if ( nLayerID == -1 )	nControl &= ~DOLAYER;
	if ( nLtypeID == -3 )	nControl &= ~DOLTYPE;
	if ( nColorID == -3 )	nControl &= ~DOCOLOR;
	if ( nControl == 0 )	return; // Nothing to do!
	if ( (nControl & DOLTYPE) && (nLtypeID <= 0) ) {
		for(i=0;i<nObject;i++) {
			data = ent_read(slist[i]);
			if ( nControl & DOLAYER )
				data.layer = nLayerID;
			if ( nControl & DOCOLOR )
				data.color = nColorID;
			if ( nControl & DOLTYPE )
				data.ltypeno = nLtypeID;
			ent_write(slist[i],data);
		}
		return;
	}
	if ( nControl & DOLTYPE )
		command("change \@slist ^MLT "+linetype(nLtypeID-1).name);
	if ( nControl & DOCOLOR )
		command("change \@slist ^MC "+prints("%d",nColorID));
	if ( nControl & DOLAYER )
		command("change \@slist ^MLA "+layer(nLayerID).name);
}
//
//	CheckObject() -- Check selected objects properties
//
void	CheckObject()
{
	int	i;
	ENTDATA data;
	for(i=0;i<nObject;i++) {
		data = ent_read(slist[i]);
		if ( i == 0 ) {
			nLayerID = data.layer;
			nColorID = data.color;
			nLtypeID = data.ltypeno;
			continue;
		}
		if ( data.layer != nLayerID )
			nLayerID = -1;	// Varies
		if ( data.color != nColorID )
			nColorID = -3;	// Varies
		if ( data.ltypeno!=nLtypeID )
			nLtypeID = -3;
		if ( (nLayerID == -1) &&
		     (nColorID == -3) &&
		     (nLtypeID == -3) )
			break;
	}
}
//
//	OpenDialog() -- Open main dialog window, create screen items.
//
void	OpenDialog()
{
	int	rBase, cBase;
	int	rInc=0x0801;
	wndopen(nMain,szText[4]/*"Change Properties"*/,0x3f1f,nMain[5]);
	rBase = 0x0800;
	cBase = 4;
	bChLayer = WndCheckBox(rBase+rInc*0,cBase,szText[5]/*"Change... "*/,0x270);
	bChColor = WndCheckBox(rBase+rInc*1,cBase,szText[5]/*"Change... "*/,0x270);
	bChLtype = WndCheckBox(rBase+rInc*2,cBase,szText[5]/*"Change... "*/,0x270);

	cBase = 16;
	bLayer	 = WndButton(rBase+rInc*0,cBase,0x108,szText[9] /*" Layer  "*/);
	bColor	 = WndButton(rBase+rInc*1,cBase,0x108,szText[10]/*" Color  "*/);
	bLtype	 = WndButton(rBase+rInc*2,cBase,0x108,szText[11]/*"Linetype"*/);
	bPick	 = WndButton(rBase+rInc*1,54,	0x108,szText[12]/*"  Pick  "*/);

	rBase += rInc*3 + 0x800;
	bResel	= WndButton(rBase,cBase,0x108,szText[13]/*" Select "*/);
	wnd_loct(rBase,cBase+10);
	wnd_puts(prints(szText[16]/*"%4d Selected"*/,nObject),0x271);

	bOK	= WndButton(rBase, 4,0x108,szText[14]/*" Do It  "*/);
	bCancel = WndButton(rBase,54,0x108,szText[15]/*" Cancel "*/);
    if ( IsWin )
	bHelp	= WndButton(rBase,42,0x108,szText[17]/*"  Help  "*/);
}
//
//	UpdateDialog() -- Update the content of the main dialog window
//
void	UpdateDialog()
{
	UpdateLayer();
	UpdateColor();
	UpdateLtype();
	wnd_fmark(bChLayer,0,(nControl&DOLAYER)?0x8000:0x80ff);
	wnd_fmark(bChLtype,0,(nControl&DOLTYPE)?0x8000:0x80ff);
	wnd_fmark(bChColor,0,(nControl&DOCOLOR)?0x8000:0x80ff);
}

void	UpdateLayer()
{
	int	rBase=0x800;
	int	rInc=0x0801;
	wnd_loct(rBase+rInc*0,25);
	wnd_puts(prints("%-24s",(nLayerID>=0)?layer(nLayerID).name:szText[6]/*"Varies..."*/),0x270);
}

void	UpdateColor()
{
	int	rBase=0x800;
	int	rInc=0x0801;
	int	cid;
	int	pencolor[16];

	wnd_loct(rBase+rInc*1,25);
	switch(nColorID) {
	  case -1:
	    wnd_puts(prints("%-12s",szText[7]/*"[ByLayer]"*/),0x270);
	    cid = layer(nLayerID).color;
	    break;
	  case -2:
	    wnd_puts(prints("%-12s",szText[8]/*"[ByBlock]"*/),0x270);
	    cid = @BYBLKCOLOR;
	    break;
	  case -3:
	    wnd_puts(prints("%-12s",szText[6]/*"Varies..."*/),0x270);
	    return;
	  default:
	    wnd_puts(prints("%-12s",prints("Color %d",nColorID)),0x270);
	    cid = nColorID;
	    break;
	}
	setdata(pencolor,@pencolor);
	wnd_puts(prints("%8s",""),(pencolor[cid&0x0f]<<4)&0xf0);
}

void	UpdateLtype()
{
	int	rBase=0x800;
	int	rInc=0x0801;
	int	dash[12];
	LTYPE	ltype;
	int	lid;
	wnd_loct(rBase+rInc*2 25);
	switch(nLtypeID) {
	  case	0:
	    wnd_puts(prints("%-12s",szText[7]/*"[ByLayer]"*/),0x270);
	    lid = ltypeid(layer(nLayerID).ltype)-1;
	    break;
	  case -1:
	    wnd_puts(prints("%-12s",szText[8]/*"[ByBlock]"*/),0x270);
	    lid = ltypeid(layer(nLayerID).ltype)-1;
	    break;
	  case -3:
	    wnd_puts(prints("%-12s",szText[6]/*"Varies..."*/),0x270);
	    return;
	  default:
	    wnd_puts(prints("%-12s",linetype(nLtypeID-1).name),0x270);
	    lid = nLtypeID;
	    break;
	}
	ltype = linetype(lid-1);
	memmove(dash,&ltype.dash[0],sizeof(dash));
	wnd_puts(DashPict(dash,28),0x71);
}
//
//	DashPict -- Get linetype pictures using dash-dot
//
DashPict(int dash[], int len)
{
	int	i,j,k,n,m,minlen;
	char	buf[128];

	minlen = len;
	for(i=0;i<12;i+=2) {
	    if ( (dash[i]==0) && (dash[i+1]==0) )
		break;
	}
	if ( i == 0 ) {
	    memset(buf,'_',minlen);
	    buf[minlen] = 0;
	    return buf;
	}

	for(k=0;k<minlen;) {
	    for(j=0;j<i;j+=2) {
		m = dash[j];
		if ( m ) {
		    if ( (k+m) > minlen )
			m = minlen - k;
		    for(n=0;n<m;n++)
			buf[k++]='_';
		}
		else
		    buf[k++] = '.';
		if ( k >= minlen )
		    break;
		m = dash[j+1];
		if ( (k+m) > minlen )
		    m = minlen - k;
		for(n=0;n<m;n++)
		    buf[k++] = ' ';
		if ( k >= minlen )
		    break;
	    }
	}
	buf[minlen] = 0;
	return buf;
}
//
//	CloseDialog() -- Close the main dialog window, save window attribute
//
void	CloseDialog()
{
	wndclose();
}
//
//	WndCheckBox() -- Create window check box
//
int	WndCheckBox(int r,int c,char text[],int atrb)
{
	int	bID;
	if ( nSupport )
		return wnd_rcbox(r,c,text,0);
	wnd_loct(r,c);
	bID = wnd_field(1,0x1f0,0x102);
	wnd_rloct(0,2);
	if ( IsWin )  atrb |= 0x800;
	wnd_puts(text,atrb);
	return bID;
}
//
int	WndButton(int r,int c, int bsize, char text[])
{
	int	i;
	if ( nSupport ) 	// Having direct support ?
		return wnd_button(r,c,bsize,0x1277,text);
	wnd_loct(r,c);
	i = wnd_field(0,IsWin?0x1277:0x370,bsize);
	wnd_puts(text,0x270);
	return i;
}

char	szAsciiText[][40]={	    // Default English Display Text
/*  0 */   "\n## No specific layer to change!",
/*  1 */   "\n## No specific linetype to change!",
/*  2 */   "\n## No specific color to change!",
/*  3 */   "\nPick properties from entity: ",
/*  4 */   "Change Properties",
/*  5 */   "Change...",
/*  6 */   "Varies...",
/*  7 */   "[ByLayer]",
/*  8 */   "[ByBlock]",
/*  9 */   "&Layer",
/* 10 */   "&Color",
/* 11 */   "Line&type",
/* 12 */   "&Pick",
/* 13 */   "&Select",
/* 14 */   "&Do It",
/* 15 */   "&Quit",
/* 16 */   "%4d Selected",
/* 17 */   "&Help",
};

char	szBig5Text[][26]={	   // Default Chinese Big-5 Display Text
/*  0 */   "\n## |wSwϼhi!",
/*  1 */   "\n## |wSwui!",
/*  2 */   "\n## |wSwCi!",
/*  3 */   "\nп˥Ϥ: ",
/*  4 */   "  S   ",
/*  5 */   "e\\:",
/*  6 */   "@...",
/*  7 */   "e̹ϼhf",
/*  8 */   "e̹ϲաf",
/*  9 */   "ϼh[&L]",
/* 10 */   "C[&C]",
/* 11 */   "u[&T]",
/* 12 */   "[&P]",
/* 13 */   "[&S]",
/* 14 */   "[&D]",
/* 15 */   "[&Q]",
/* 16 */   "@%4d ӹϤ",
/* 17 */   "[&H]",
};

char	szGB2312Text[][26]={	// Default Chinese GB2312 Display Text
/*  0 */   "\n## δָضͼи!",
/*  1 */   "\n## δָضͽи!",
/*  2 */   "\n## δָضɫи!",
/*  3 */   "\nѡȡͼԪ: ",
/*  4 */   "ͼ Ԫ    ",
/*  5 */   ":",
/*  6 */   "һ...",
/*  7 */   "ͼ",
/*  8 */   "ͼ",
/*  9 */   "ͼ[&L]",
/* 10 */   "ɫ[&C]",
/* 11 */   "[&T]",
/* 12 */   "ȡ[&P]",
/* 13 */   "ѡȡ[&S]",
/* 14 */   "ִ[&D]",
/* 15 */   "[&Q]",
/* 16 */   "%4d ͼԪ",
/*  2 */   "˵[&H]",
};
//
//	The following Japenese messages are translated by _____________.
//
char	szSJISText[][40]={    // Default Japenese SJIS Display Text
/*  0 */   "\n## C[͕ύXł܂!",
/*  1 */   "\n## ͕ύXł܂!",
/*  2 */   "\n## J[͕ύXł܂!",
/*  3 */   "\n}`̓sbNĉ: ",
/*  4 */   "ύX܂B",
/*  5 */   "ύX...",
/*  6 */   "ύX...",
/*  7 */   "C[",
/*  8 */   "ubN",
/*  9 */   "C[",
/* 10 */   "J[",
/* 11 */   "",
/* 12 */   "߯ ",
/* 13 */   "}`I",
/* 14 */   "s[&D]",
/* 15 */   "ݾ[&Q]",
/* 16 */   "%d }`I",
/* 17 */   "wv",
};
//
//	The following Korean messages are translated by _____________.
//
char	szKoreanText[][40]={	// Default Korean Display Text
/*  0 */   "\n## No specific layer to change!",
/*  1 */   "\n## No specific linetype to change!",
/*  2 */   "\n## No specific color to change!",
/*  3 */   "\nPick property from entity: ",
/*  4 */   "Change Properties",
/*  5 */   "Change...",
/*  6 */   "Varies...",
/*  7 */   "[ByLayer]",
/*  8 */   "[ByBlock]",
/*  9 */   "&Layer",
/* 10 */   "&Color",
/* 11 */   "&Linetype",
/* 12 */   "&Pick",
/* 13 */   "&Select",
/* 14 */   "&Do It",
/* 15 */   "&Quit",
/* 16 */   "%4d Selected",
/* 17 */   "&Help",
};

char	szDosAsciiText[][40]={	       // Default English Display Text
/*  0 */   "\n## No specific layer to change!",
/*  1 */   "\n## No specific linetype to change!",
/*  2 */   "\n## No specific color to change!",
/*  3 */   "\nPick property from entity: ",
/*  4 */   "Change Properties",
/*  5 */   "Change...",
/*  6 */   "Varies...",
/*  7 */   "[ByLayer]",
/*  8 */   "[ByBlock]",
/*  9 */   " Layer  ",
/* 10 */   " Color  ",
/* 11 */   "Linetype",
/* 12 */   "  Pick  ",
/* 13 */   " Select ",
/* 14 */   " Do It  ",
/* 15 */   "  Quit  ",
/* 16 */   "%4d Selected",
/* 17 */   "  Help  ",
};
char	szDosBig5Text[][26]={	      // Default Chinese Big-5 Display Text
/*  0 */   "\n## |wSwϼhi!",
/*  1 */   "\n## |wSwui!",
/*  2 */   "\n## |wSwCi!",
/*  3 */   "\nп˥Ϥ: ",
/*  4 */   "  S   ",
/*  5 */   "e\\:",
/*  6 */   "@...",
/*  7 */   "e̹ϼhf",
/*  8 */   "e̹ϲաf",
/*  9 */   "  ϼh  ",
/* 10 */   "  C  ",
/* 11 */   "  u  ",
/* 12 */   "    ",
/* 13 */   "    ",
/* 14 */   "    ",
/* 15 */   "    ",
/* 16 */   "@%4d ӹϤ",
/* 17 */   "  Help  ",
};

char	szDosGB2312Text[][26]={    // Default Chinese GB2312 Display Text
/*  0 */   "\n## δָضͼи!",
/*  1 */   "\n## δָضͽи!",
/*  2 */   "\n## δָضɫи!",
/*  3 */   "\nѡȡͼԪ: ",
/*  4 */   "ͼ Ԫ    ",
/*  5 */   ":",
/*  6 */   "һ...",
/*  7 */   "ͼ",
/*  8 */   "ͼ",
/*  9 */   "  ͼ  ",
/* 10 */   "  ɫ  ",
/* 11 */   "    ",
/* 12 */   "  ȡ  ",
/* 13 */   "  ѡȡ  ",
/* 14 */   "  ִ  ",
/* 15 */   "    ",
/* 16 */   "%4d ͼԪ",
/* 17 */   "  Help  ",
};
//
//	The following Japenese messages are translated by _____________.
//
char	szDosSJISText[][40]={	 // Default Japenese SJIS Display Text
/*  0 */   "\n## C[͕ύXł܂!",
/*  1 */   "\n## ͕ύXł܂!",
/*  2 */   "\n## J[͕ύXł܂!",
/*  3 */   "\n}`̓sbNĉ: ",
/*  4 */   "ύX܂B",
/*  5 */   "ύX...",
/*  6 */   "ύX...",
/*  7 */   "C[",
/*  8 */   "ubN",
/*  9 */   "C[",
/* 10 */   " J[ ",
/* 11 */   "    ",
/* 12 */   " sbN ",
/* 13 */   "}`I",
/* 14 */   "  s  ",
/* 15 */   "  ݾ ",
/* 16 */   "%d }`I",
/* 17 */   " wv ",
};
//
//	The following Korean messages are translated by _____________.
//
char	szDosKoreanText[][40]={    // Default Korean Display Text
/*  0 */   "\n## No specific layer to change!",
/*  1 */   "\n## No specific linetype to change!",
/*  2 */   "\n## No specific color to change!",
/*  3 */   "\nPick property from entity: ",
/*  4 */   "Change Properties",
/*  5 */   "Change...",
/*  6 */   "Varies...",
/*  7 */   "[ByLayer]",
/*  8 */   "[ByBlock]",
/*  9 */   " Layer  ",
/* 10 */   " Color  ",
/* 11 */   "Linetype",
/* 12 */   "  Pick  ",
/* 13 */   " Select ",
/* 14 */   " Do It  ",
/* 15 */   "  Quit  ",
/* 16 */   "%4d Selected",
/* 17 */   "  Help  ",
};

#define BIG5		0x404
#define GB2312		0x804
#define SJIS		0x411
#define KOREAN		0x412
#define ASCII		0x409

#define TGBIG5		1
#define TGGB2312	2
#define TGSJIS		5
#define TGKOREAN	3

int	SetMsgText()
{
	char	szBuf[80];
	int	switched;

	IsWin = 1;
	switch(1) {  // Test if we are in TwinCAD ...
	  case !identify("os_type"):
	  case (os_type()<0):	// Must be in DOS version!
		IsWin = 0;
		break;
	}

	if ( IsWin ) {
	    szBuf = get_profile("General Setup","AlternateLanguage");
	    switched = 0;
	    if ( szBuf!="" )
		switched = eval(szBuf);
	    if ( switched ) {
		switch(os_language()) {
		  case BIG5:	setdata(szText,szBig5Text);	break;
		  case GB2312:	setdata(szText,szGB2312Text);	break;
		  case SJIS:	setdata(szText,szSJISText);	break;
		  case KOREAN:	setdata(szText,szKoreanText);	break;
		  default:	setdata(szText,szAsciiText);	break;
		}
	    }
	    else
		setdata(szText,szAsciiText);
	}
	else {	// Switched or not, depends on TGF type in DOS version
	    switch(@tgftype) {
	      case TGBIG5:    setdata(szText,szDosBig5Text);	 break;
	      case TGGB2312:  setdata(szText,szDosGB2312Text);	 break;
	      case TGSJIS:    setdata(szText,szDosSJISText);	 break;
	      case TGKOREAN:  setdata(szText,szDosKoreanText);	 break;
	      default:	      setdata(szText,szDosAsciiText);	 break;
	    }
	}
	return 0;
}
//
//	End of this Program file
//
